# KNMI2CABOWE3b.py
# P.A.J. van Oort 19 Dec 2018
# Python script
#
# reads offical KNMI weather data file from website http://www.knmi.nl/klimatologie/daggegevens/download.html
# breaks it apart into annual weather files for input into Wageningen Crop Models like Wofost, LINTUL, SUCROS, GECROS
# adds the proper header for annual weather files
# checks if radiation data are available
# if not then checks if sunshine hours are available and calculates radiation using regression equation from data of station De Bilt:
# RRDreg = 288.6 + 15.74*SQ (R2 = 0.74)
# if also no sunshine hours available then no annual weather file is generated
# writes value -99 (missing) for missing data
# you need to manually fill in some station metadata in stndict below:
# for example stndict = {"etmgeg_310.txt":('Vlissingen','51.45','3.60','8.0'), means that etmgeg_310.txt is the weather data file of station Vlissingen
# latitude 51.45, longitude 3.6, height of station: 8.0 m.a.s.l.
# station metadata are in http://projects.knmi.nl/klimatologie/metadata/stationslijst.html
# writes -0.18 -0.55 for Angstrom parameters. Negative number means file contains radiation data, so Angstrom parameters are not used


import os
import math
stnfiledir = "D:/weather/knmi/"
outfiledir = "D:/weather/CABO/"
stnfilelist = ["etmgeg_280.txt"]
# LAT (N) in decimal degrees, LON (E) in dd, altitude (m)
stndict = {"etmgeg_356.txt":('Herwijnen','51.51','5.09','0.7'),"etmgeg_283.txt":('Hupsel','52.04','6.39','29.7'),"etmgeg_310.txt":('Vlissingen','51.45','3.60','8.0'),"etmgeg_260.txt":('De Bilt','52.1','5.183','2.0'),"etmgeg_280.txt":('Eelde','53.13','6.58','3.5'),"etmgeg_269.txt":('Lelystad','52.45','5.52','-4.2')}
missing = -99.
RDDcol = 20 #Q         = Globale straling (in J/cm2) / Global radiation (in J/cm2)
TMMNcol = 12 #TN        = Minimum temperatuur (in 0.1 graden Celsius) / Minimum temperature (in 0.1 degrees Celsius)
TMMXcol = 14 #TX        = Maximum temperatuur (in 0.1 graden Celsius) / Maximum temperature (in 0.1 degrees Celsius)
UNcol = 38 #UN        = Minimale relatieve vochtigheid (in procenten) / Minimum relative atmospheric humidity (in percents)
WNcol = 4 #FG        = Etmaalgemiddelde windsnelheid (in 0.1 m/s) / Daily mean windspeed (in 0.1 m/s) 
RAINcol = 22 #RH        = Etmaalsom van de neerslag (in 0.1 mm) / Daily precipitation amount (in 0.1 mm) (-1 voor/for <0.05 mm)
SQcol = 18 #SQ        = Zonneschijnduur (in 0.1 uur) berekend uit de globale straling (-1 voor <0.05 uur) / Sunshine duration (in 0.1 hour) calculated from global radiation (-1 for <0.05 hour)

for etmgegfilename in stnfilelist:
    etmgegfile = open(stnfiledir+etmgegfilename,"r")
    etmgeglines = etmgegfile.readlines()
    STN = etmgegfilename[7:10]
    iyyyyprevrec = 0
    Qflag = 0; SQflag = 0
    for j in range(49,len(etmgeglines)):
#    for j in range(21000,22000):
        etmgegsplit = etmgeglines[j].split(",")
        yyyy = etmgegsplit[1][0:4]
        iyyyy = int(yyyy)
        if iyyyy != iyyyyprevrec:
            try:
                outfile.close()
                if Qflag == 0:
                    if SQflag == 0:
                        print(outfilename)
                    else:
                        print(outfilename + "; radiation estimated from sunshine hours")
                else:
                    print("deleted " + outfilename+" - missing radiation data")
                    os.remove(stnfiledir+outfilename)
            except:
                print("running ...")
            Qflag = 0; SQflag = 0
            idoy = 1
            outfilename = "NL"+STN+"."+yyyy[1:4]
            outfile = open(outfiledir+outfilename,"w")
            outfile.write("*---------------------------------------------------------*"+"\n"+
                          "*   Station: "+stndict.get(etmgegfilename)[0]+"\n"+
                          "*      Year: "+yyyy+"\n"+
                          "*    Source: "+etmgegfilename+"\n"+
                          "*    Author: "+"http://www.knmi.nl/klimatologie/daggegevens/download.html"+"\n"+
                          "* Longitude: "+stndict.get(etmgegfilename)[2]+"\n"+
                          "*  Latitude: "+stndict.get(etmgegfilename)[1]+"\n"+
                          "* Elevation: "+stndict.get(etmgegfilename)[3]+"\n"+
                          "*  Comments: "+"Generated with python script KNMI2CABOWE3.py (script by Pepijn van Oort)"+"\n"+
                          "*"+"\n"+
                          "*  Columns:"+"\n"+
                          "*  ========"+"\n"+
                          "*  1. station number"+"\n"+
                          "*  2. year"+"\n"+
                          "*  3. day of year"+"\n"+
                          "*  4. irradiation (kJ m-2 d-1)"+"\n"+
                          "*  5. minimum temperature (degrees Celsius)"+"\n"+
                          "*  6. maximum temperature (degrees Celsius)"+"\n"+
                          "*  7. vapour pressure (kPa)"+"\n"+
                          "*  8. mean wind speed (m s-1)"+"\n"+
                          "*  9. precipitation (mm d-1)"+"\n"+
                          "*  \n"+
                          "** WCCDESCRIPTION=Netherlands, "+stndict.get(etmgegfilename)[0]+"\n"+
                          "** WCCFORMAT=2"+"\n"+
                          "** WCCYEARNR="+yyyy+"\n"+
                          "*---------------------------------------------------------*"+"\n"+
                          " "+stndict.get(etmgegfilename)[2]+" "+stndict.get(etmgegfilename)[1]+" "+stndict.get(etmgegfilename)[3]+" -0.15 -0.55"+"\n")
        else:
            idoy = idoy + 1
        iyyyyprevrec = iyyyy
#*  irradiation (kJ m-2 d-1)"+"\n",
#*     RDD    Daily global radiation        J/m2/d
        try:
            RDDKNMI = float(etmgegsplit[RDDcol])
            RDD = RDDKNMI*0.001*(100.*100.) #J/cm2 -> kJ/m2  
        except:
            try:
                SQflag = 1
                SQKNMI = float(etmgegsplit[SQcol])
                if SQKNMI == -1.:
                    RDD = missing
                    Qflag = 1
                else:                   
                    RDDKNMI = 288.6 + 15.74*SQKNMI               
                    RDD = RDDKNMI*0.001*(100.*100.) #J/cm2 -> kJ/m2 
            except:
                RDD = missing
                Qflag = 1
#*  minimum temperature (degrees Celsius)
#*     TMMN   Daily minimum temperature     degree C
        try:
            TMMNKNMI = float(etmgegsplit[TMMNcol])
            TMMN = TMMNKNMI*0.1 #0.1oC -> oC  
        except:
            TMMN = missing
#*  maximum temperature (degrees Celsius)
#*     TMMX   Daily maximum temperature     degree C
        try:
            TMMXKNMI = float(etmgegsplit[TMMXcol])
            TMMX = TMMXKNMI*0.1 #0.1oC -> oC  
        except:
            TMMX = missing
#*  vapour pressure (kPa)
#VP = (RHmin/100)*VPS(Tmax) = minimum relative humidity (%) / saturated vapour pressure (kPa) at Tmax (oC)
#Equation below from Goudriaan, J., van Laar, H.H., 1994. Modelling potential crop growth processes. In: Current Issues in Production Ecology. Kluwer Academic Publishers, Dordrecht, The Netherlands, pp. 239p.
#*     VP     Vapour pressure               kPa
        if (TMMX !=missing):
            try:
                UNKNMI = float(etmgegsplit[UNcol])
                VPS = 0.6017*math.exp(17.4*TMMX/(239+TMMX))
                VP = (UNKNMI/100)*VPS
            except:
                VP = missing
        else:
            VP = missing
#*  mean wind speed (m s-1)
#*     WN     Wind speed                    m/s
        try:
            WNKNMI = float(etmgegsplit[WNcol])
            WN = WNKNMI*0.1 #0.1 m/s -> m/s  
        except:
            WN = missing
#*  precipitation (mm d-1)
#*     RAIN   Precipitation                 mm
        try:
            RAINKNMI = float(etmgegsplit[RAINcol])
            if RAINKNMI == -1.:
                RAIN = 0.
            else:
                RAIN = RAINKNMI*0.1 #0.1 mm -> mm
        except:
            RAIN = missing
        outfile.write(STN + " "+yyyy+" "+str(idoy)+" "+str(round(RDD,0))+" "+str(round(TMMN,1))+" "+str(round(TMMX,1))+" "+str(round(VP,3))+" "+str(round(WN,1))+" "+str(round(RAIN,1))+"\n")
    etmgegfile.close()
outfile.close()

# print finished
print("finished, output written to: " + outfiledir)
